# iOS Slate+ Expand Samples with Riiiver

This is a sample app with which you can learn how to integrate the Riiiver SDK into your companion app for Slate+.
With this app you'll find:

- Expansion of UX with Slate+ by utilizing the Riiiver SDK.
- How the Riiiver functions, piece(s) and iiidea(s), work.
- User Interface provided by our SDK, e.g. login screen, iiidea list screen.

## How does this sample demonstrate?

- (1) Scan BLE devices nearby.
- (2) Connect to Slate+ evaluation board(EVB).
  - Discover the service and characteristics provided as `Riiiver_Demo` by QCLI.
  - If it doesn't find the service and characteristics, it doesn't move to forward pages.
  
- (3) Login to Riiiver.
- (4) Complete iiidea settings.
- (5) Assign the Riiiver functions that'll be triggered by pushing the button on the EVB.
  - Push the button and find the result that appears on the display of the EVB.


### Screenshots

![slate+_sample_app_journey](./readme_rsc/screenshots/app_journey.PNG)

## Out of scope for this Sample App

- This app is not a template. Integrate our SDK to your app as you like, e.g. you can choose a suitable architecture for your app.
- This app does not implement data storage that keeps the app state even after killing and relaunching the app.
- This app does not implement a complete paring of BLE connection. The Riiiver functions only work when the connection is alive between Slate+ and your mobile phone.

## Prepare for development using Riiiver SDK

### Preparing for development

#### What this section describes:

- Sign up as a developer
- [resource file] `erconfiguration.json`.

To develop with Riiiver SDK, sign up as a developer and register information about your service and company. Please visit our developer site and sign up by following the instructions.

[Riiiver Developer Site](https://dev.riiiver.com/)

The required information is as follows:
- Your company information.
- Your application information.
- Your device information.

Riiiver SDK reads vendor and app information by referring to the `erconfiguration.json` file. The `VendorId` and `AppId` should be the IDs that Riiiver has given you when you registered your vendor and app information. 

The prepared `erconfiguration.json` file must be located in the root hierarchy of the bundle. In this project, the file is placed in the directory `Infra/RiiiverSDK/SupportingFiles`, but Xcode's Run Script copies the file to the root of the bundle when the build is executed.

### Install the Riiiver SDK

#### What this section describes:

- How to import the `RiiiverSDK.framework` into your project

In this sample project, we use [CocoaPods](https://cocoapods.org/) to install Riiiver SDK. We have provided a Podfile in the root directory of the project. Please install Riiiver SDK by yourself with the below command.

```shell
$ pod install
```

After the installation is completed, you will find the RIiiver SDK and external libraries used in this project under the ``Pods/`` directory, and a ``{project_name}.xcworkspace`` file in the root directory. When building the application, please open and build the .xcworkspace file.

##### note:

The `Pods/` directory and Xcode workspace are not subject to git management.

## Using Riiiver SDK features

### Initialize the Riiiver SDK

#### What this section describes:

- [method] `initialize(deviceIdList:)`
- [method] `addListener(listener:)`
- [protocol] `ERSdkListener`

Be sure to execute the `initialize(deviceIdList:)` method before using any of the methods provided in the Riiiver SDK.

In this sample app, we have prepared the initialization method in the class `RiiiverSDKClient` for calling methods of Riiiver SDK, and that method is called every time before executing a method of Riiiver SDK. The implementation is designed to return immediately if it has already been initialized, so it does not matter how many times it is called. The argument is passed the ID of the device supported by the application as an Array of String. If you have not registered your device information, please visit [our developer site](https://dev.riiiver.com/) and register it first.

Also, since the `ERSdkListener` protocol is provided to delegate processing to the app side when an event occurs in the Riiiver SDK, we use the `addListener(listener:)` method to set the delegate after initialization is complete. This method is also implemented to do nothing if the delegate instance is the same, so it does not matter how many times it is called.

```swift
/* qcc5100.sample/Infra/RiiiverSDK/RiiiverSDKClient.swift */
private func initialize() async throws -> Void {
    let error = await RiiiverSdkManager.initialize(deviceIdList: RiiiverConstants.deviceIds)
    if let error {
        print("## RiiiverSdkClient: initialized failed due to \(error.localizedDescription)")
        throw error
    }
    RiiiverSdkManager.addListener(delegate: self) // ← add delegate
    return
}
```

The `ERSdkListener` protocol provides the following delegate methods, so please implement the necessary processing.

##### Methods of `ERSdkListener`

- `startApplet(applet:error:)`

  Called when the execution of iiidea is started.

- `endApplet(applet:blockName:output:error:)`

  Called when the execution of iiidea has finished. `blockName` is the type of Piece (trigger, service, or action) that terminated. In case of abnormal termination, it will tell which Piece has been reached. Also, `error` contains error information.

- `onLogin(applets:)`

  Called when login to Riiiver is completed.

- `onLogout()`

  Called when users log out of Riiiver.

##### note:

- `Applet` is the name of iiidea when developing Riiiver. Please read "iiidea" instead of "applet" as it is used in the code and documentation.
- `Block` is the name of piece when developing Riiiver. Please read "piece" instead of "block" as it is used in the code and documentation.

### Use the authentication features(login and out)

#### What this section describes:

- [method] `getLoginUrl()`
- [method] `verifyCodeOf(uri:)`
- [method] `logout()`

An authentication page is provided to login to Riiiver. Please open the URL in the return value of the `getLoginUrl()` method. When a user completes the login operation on the authentication page, a redirect will occur. The redirect URL is registered as a part of your application information via [our developer site](https://dev.riiiver.com/). If you have not registered, please visit [our developer site](https://dev.riiiver.com/) and register the redirect URL. Please set it in `info.plist` so that the app can be launched via a custom scheme.
When the app is launched via a custom scheme, a short-lived authorization code is given as a query parameter in the URL. The URL retrieved by the `application(_:open:options:)` delegate method in the `AppDelegate` or the `onOpenURL(perform:)` method if you are using SwiftUI, is passed to the `verifyCodeOf(uri:)` method to complete the login to Riiiver.

##### note:

- It is recommended to use `SFSafariViewController` to open the authentication page, as it will cause a redirect and pass values to the app with a custom scheme. If you use the `UIApplication.shared.open(url:)` method to open in an external browser, the user may have changed their default browser from Safari.

If you want to log out of Riiiver, execute the `logout()` method at any time.

### Display the iiidea list and the settings screen

#### What this section describes:

- [class] `ERUserWebPagesView`
- [protocol] `ERUserWebPagesViewDelegate`

We provide a web page that aggregates a iiidea store for users to search for and obtain iiidea, and a list view for users to set or delete already obtained iiidea. The `ERUserWebPagesView` class, which extends the `WKWebView` class, is provided as a class for displaying these web pages.

`loadPage(type:)` method can be executed to display the page, in the `viewDidLoad()` method when using UIViewController in your project, or in the `makeUIView(context:)` method when using SwiftUI's UIViewRepresentable in your project.

Some iiidea require the use of device's permissions or OAuth authentication for third-party services at runtime, so when a user requests those permissions on a web page, the `ERUserWebPagesViewDelegate` protocol is provided to delegate the handling of those events to the app side. The following delegate methods are provided for this delegate, so please implement the necessary processing.

##### Methods of `ERUserWebPagesViewDelegate`.

- `didReceivePermissionRequest(permissionType:)`

  Called when the request button for a device's permission on a web page is tapped. The type of permission being requested is passed as an argument, so that you can present the user with the permission's usage authorization dialog.

- `didReceiveOpenUrlRequest(url:)`

  Called when a new URL needs to be opened, such as for OAuth authentication. Open the URL passed as argument.

- `didReceiveCloseRequest()`

  Called when the close button on the web page is tapped.

- `onError(error:)`

  Called when an error occurs on a web page.

Once the necessary settings for iiidea have been completed, iiidea can be activated. iiidea is activated and is ready to run.

### Pre-install some iiidea(s)

#### What this section describes:

- [method] `forcedDownloadApplet(appletIds:)`

If you want to make pre-installed some features when the user installs your application, you can use the `forcedDownloadApplet(appletIds:)` method to make the iiidea of the ID specified in the argument already obtained.

### Get the available iiideas list

#### What this section describes:

- [method] `getMyApplets(deviceIdList:)`
- [class] `ERApplet`

This sample app is implemented so that each of the two physical buttons on the Slate+ evaluation board can execute a different iiidea when pressed. For this purpose, the app provides a UI that allows the user to freely select the iiidea to be executed. At this time, the method `getMyApplets(deviceIdList:)` is used to retrieve the list of iiidea already obtained by the user. By passing a device ID as an argument in the form of an Array of String, a list of available iiideas on that device can be obtained. The return value is an Array of `ERApplet`.

The `ERApplet` class provides properties containing iiidea metadata, as well as methods related to activation deactivation and iiidea execution. When you want to find out if a iiidea is ready to run (activated state), you can call `getActivateStatus()` method.

### Implement the Piece(s) and execute iiidea

#### What this section describes:

- [protocol] `ERBlockExecutor`
- [protocol] `ERBlockDelegate`

iiidea is a small application that combines 3 Pieces. We call Trigger Piece for the first one, Service Piece for the second one, and Action Piece for the third one, respectively.

All Pieces implemented in the app or SDK must inherit the `ERBlockExecutor` protocol. The following methods are defined in this protocol, so please implement the necessary processing.

##### Methods of `ERBlockExecutor`

- `enable(applet:)`

  Called when an `applet` using this Piece is activated by the user. Return true if the piece can be activated, or false if it cannot be activated.

  Although there is no example implementation in this sample app, the geofence trigger implemented in the SDK, for example, checks whether permission to use location information has been obtained and returns false if permission has not been obtained.

- `execute(applet:blockName:preferenceJson:inputJson:handler:)`  

  Called when an `applet` that uses this Piece is executed and in the order of execution of this Piece. In other words, this method is called when the previous Piece is completed.
  When the processing of the Piece is completed, the `completed(outputJson:error:)` method of the `ERBlockDelegate` passed as the `handler` is executed. In case of abnormal termination, call the `completed(outputJson:error:)` method with the argument `error`.
  Note that this method is called when it turns activated in the case of the first Piece of iiidea, the trigger Piece. It is necessary to make sure that the execution of iiidea starts when the desired event actually occurs. Regarding the `QCC5100TriggerPieceCore`, which fires when a button on the Slate+ EVB is pressed by the user, the `QCC5100TriggerPieceCore` instance holds the iiidea id and a `handler` for it as a key-value pair in the dictionary type. And it implements the `didTrigger(appletId:input:)` method to notify events. At the timing when the `peripheral(_:didUpdateValueFor:error:)` method of `CBPeripheralDelegate` is called (i.e., when the app is notified of the button press event on the EVB), the `didTrigger( appletId:input:)` method is called to start iiidea execution.

- `disable(applet:)`

  Called when an `applet` that uses this Piece is deactivated. Implement a process such as releasing resources that needed to be held while waiting for execution.

  In `QCC5100TriggerPieceCore`, the `ERBlockDelegate` corresponding to iiideaId is discarded. Also, the geofence trigger implemented in the SDK stops monitoring location information.

### Run iiidea from outside the app using Firebase

#### What this section describes:.

- [method] `setNotificationToken(_:)`
- [method] `executeTaskForRemoteNotification(_:)`

##### Preparation

The Firebase Cloud Messaging API is used to achieve this. The following files are required to use the function.

- GoogleService-info.plist

- APNs authentication key


For information on how to prepare the file, please refer to the following document

[Registering a key](https://firebase.flutter.dev/docs/messaging/apple-integration#1-registering-a-key)

[Registering an App Identifier](https://firebase.flutter.dev/docs/messaging/apple-integration#2-registering-an-app-identifier)

[Generating a provisioning profile](https://firebase.flutter.dev/docs/messaging/apple-integration#3-generating-a-provisioning-profile )

[Enable Push Notifications](https://firebase.flutter.dev/docs/messaging/apple-integration#enable-push-notifications)

[Enable Background Modes (iOS only)](https://firebase.flutter.dev/docs/messaging/apple-integration#enable-background-modes-ios-only)

The prepared `Google-Service-info.plist` file must be placed in the root hierarchy of the bundle. In this project, the file is placed in the directory `Infra/Firebase/GoogleService`, but Xcode's Run Script copies the file to the root of the bundle at build execution.

##### FirebaseMessaging Setup

In order to use the FirebaseMessaging APIs, some implementation is required in the AppDelegate.
In the `application(_:didFinishLauchingWithOptions:)` method of the AppDelegate, you must perform these actions: (1) the FCM token issue request, (2) FCM delegate configuration, (3) push notification permission request, and (4) APNs device token issue request in the `application(_:didFinishLauchingWithOptions:)` method of AppDelegate.

```Swift
// (1) Request to issue FCM token and set delegates
FirebaseApp.configure()     
// (2) Set messaging delegate
Messaging.messaging().delegate = self
        
// Register for remote notifications.
// (3) This shows a permission dialog on first run, to show the dialog at
// a more appropriate time move this registration accordingly.
if #available(iOS 10.0, *) {
    // For iOS 10 display notification (sent via APNS)
    UNUserNotificationCenter.current().delegate = self

    let authOptions: UNAuthorizationOptions = [.alert, .badge, .sound]
    UNUserNotificationCenter.current().requestAuthorization(
          options: authOptions,
          completionHandler: { _, _ in }
    )
} else {
    let settings: UIUserNotificationSettings = UIUserNotificationSettings(types: [.alert, .badge, .sound], categories: nil)
    application.registerUserNotificationSettings(settings)
}

// (4) Request to issue APNs device token
application.registerForRemoteNotifications()
```

Implement the `application(_:didRegisterForRemoteNotificationsWithDeviceToken:)` method, which will be called when the APNs device token is issued. The FCM token is issued when the device token for the APNs is registered with the Firebase Messaging SDK.

```Swift
func application(_ application: UIApplication,
                 didRegisterForRemoteNotificationsWithDeviceToken deviceToken: Data) {
    debugPrint("[APNs]デバイストークン受信 [\(deviceToken.map{ String(format: "%.2hhx", $0) }.joined())]")
    Messaging.messaging().apnsToken = deviceToken
}
```

Inherit the `MessagingDelegate` protocol and implement the `messaging(_:didReceipRegistrationToken:)` method, which will be called when the FCM token is issued, and set the FCM token to the Riiiver SDK. Execute the `setNotificationToken(_:)` method.

```Swift
extension AppDelegate: MessagingDelegate {
    func messaging(_ messaging: Messaging, didReceiveRegistrationToken fcmToken: String?) {
        if let fcmToken {
            print("fcmToken = \(String(describing: fcmToken))")
            self.fcmToken = fcmToken
            Task(priority: .medium) {
                do {
                    try await RiiiverSDKClient.sharedInstance.setNotificationToken(fcmToken)
                } catch (let error) {
                    print("## MessagingDelegate setNotificationToken failed due to \(error.localizedDescription)")
                }
            }
        } else {
            debugPrint("fcm token is nil.")
            return
        }
    }
}
```

Implement the `application(_:didReceiveRemoteNotification:fetchCompletionHandler:)` method, which is called when a data message is received using Firebase ([cf. message type](https://firebase.google.com/docs/cloud-messaging/concept-options?hl=ja#notifications_and_data_messages)). The received push notification is passed to the Riiiver SDK to delegate subsequent processing. Use the `ExecuteTaskForRemoteNotification(_:)` method.

For Riiiver-based data messages, the `userInfo` contains the classification information of the push notification with the key `"notification_id"`. If your project already uses Firebase, you can branch the process depending on the presence or absence of this key. In other words, determine if this key is included and execute the `executeTaskForRemoteNotification(_:)` method only if it is included.

## App Architecture

[VIPER](https://cheesecakelabs.com/blog/ios-project-architecture-using-viper/) + [SwiftUI](https://developer.apple.com/xcode/swiftui/)

---



# iOS Slate+ Expand Samples with Riiiver

これは、Riiiver SDKをSlate+のコンパニオンアプリに統合する方法を学ぶためのサンプルアプリです。
このアプリを使えば、以下のことがわかります：

- Riiiver SDKを活用したSlate+のUX拡張。
- Riiiverの機能（Piece、iiidea）の仕組み。
- Riiiver SDKが提供するユーザーインターフェース（ログイン画面、iiidea一覧画面など）。

## アプリのデモ操作について

- (1) 近くのBLEデバイスをスキャンします。
- (2) Slate+評価ボード(EVB)に接続します。
  - QCLIで`Riiiver_Demo`として提供されるサービスとキャラクタリスティクスを検出します。
  - 対象のサービスとキャラクタリスティクスが見つからなかった場合、次の画面に遷移しません。

- (3) Riiiverにログインします。
- (4) iiideaの設定をします。
- (5) EVB上のボタンを押した時にトリガーされるRiiiverの機能を割り当てます。
  - EVB上のボタンを押し、EVBのディスプレイに表示される結果を確認します。


### スクリーンショット

![slate+_sample_app_journey](./readme_rsc/screenshots/app_journey.PNG)

## サンプルアプリのスコープ対象外とするもの

- これはアプリのテンプレートではありません。
- アプリを終了・再起動しても、アプリの状態を保持するデータストレージ機能は実装されていません。
- このサンプルアプリではペアリングをサポートしていません。Riiiverの機能は、Slate+と接続されているときにのみ動作します。

## Riiiver SDKを用いた開発の準備

### 開発の準備をする

#### この節で説明するもの:

- 開発者登録の手順
- [resource file] `erconfiguration.json`

Riiiver SDKを利用したアプリを開発するには開発者登録および開発者情報の登録が必要です。以下のドキュメントを参照して登録を行なってください。

[開発者として登録する](https://developer.riiiver.com/docs/introduction/1324/)

[Riiiverに開発者情報とデバイス/アプリの必要な情報を登録する](https://developer.riiiver.com/docs/riiiversdk-usage/3546/)

上記で登録したベンダー情報・アプリ情報をRiiiver SDKから参照するために`erconfiguration.json`ファイルを用意します。`VendorId`, `AppId`は、ベンダー情報・アプリ情報を登録する際にRiiiverから払い出されたIDを使用してください。Riiiver SDKから読み取って使用するため、ファイル名は指定通りである必要があります。また、用意した`erconfiguration.json`ファイルは、バンドルのルート階層に配置されている必要があります。本プロジェクトでは`Infra/RiiiverSDK/SupportingFiles`というディレクトリ内に配置してありますが、XcodeのRun Scriptでビルド実行時にバンドルのルートにファイルをコピーしています。

### Riiiver SDKをインストールする

#### この節で説明するもの:

- `RiiiverSDK.framework`をプロジェクトに取り込む方法

このサンプルプロジェクトでは、[CocoaPods](https://cocoapods.org/)を使ってRiiiver SDKをインストールしています。プロジェクトのルートディレクトリにPodfileを用意しているので、各自Riiiver SDKをインストールしてください。

```shell
$ pod install
```

インストールが完了すると、`Pods/`ディレクトリ以下に、RIiiver SDKおよびこのプロジェクトで利用している外部ライブラリが、また、ルートディレクトリに`{project_name}.xcworkspace`ファイルが生成されます。アプリをビルドする際は、.xcworkspaceファイルを開いてビルドしてください。

なお、`Pods/`ディレクトリやXcode workspaceはgit管理の対象外としています。

## Riiiver SDKの機能を使用する

### Riiiver SDKをイニシャライズする

#### この節で説明するもの:

- [method] `initialize(deviceIdList:)`
- [method] `addListener(listener:)`
- [protocol] `ERSdkListener`

Riiiver SDKに用意されているメソッドを使用する前に、必ず`initialize(deviceIdList:)`メソッドを実行してください。

本サンプルアプリでは、Riiiver SDKのメソッドを呼び出すためのクラス`RiiiverSDKClient`にイニシャライズ用のメソッドを用意し、Riiiver SDKのメソッド実行前に毎回呼び出すようにしています。初期化済みであれば即リターンされる実装になっているため、何度呼び出しても問題ありません。引数には、アプリがサポートするデバイスのIDをStringのArrayで渡します。デバイス情報の登録は[Riiiverに開発者情報とデバイス/アプリの必要な情報を登録する](https://developer.riiiver.com/docs/riiiversdk-usage/3546/)を参照してください。

また、Riiiver SDK内でのイベント発生時の処理をアプリ側に委譲するための`ERSdkListener`プロトコルが用意されているので、初期化完了後に`addListener(listener:)`メソッドでdelegateを設定するようにしています。このメソッドについても、delegateのインスタンスが同一のものであれば何もしない実装になっているため、何度呼び出しても問題ありません。

```swift
/* qcc5100.sample/Infra/RiiiverSDK/RiiiverSDKClient.swift */
private func initialize() async throws -> Void {
    let error = await RiiiverSdkManager.initialize(deviceIdList: RiiiverConstants.deviceIds)
    if let error {
        print("## RiiiverSdkClient: initialized failed due to \(error.localizedDescription)")
        throw error
    }
    RiiiverSdkManager.addListener(delegate: self) // ← delegateを設定
    return
}
```

`ERSdkListener`プロトコルには以下のデリゲートメソッドが用意されていますので、必要な処理を実装してください。

##### `ERSdkListener`のメソッド

- `startApplet(applet:error:)`

  iiideaの実行が開始された際に呼び出されます。

- `endApplet(applet:blockName:output:error:)`

  iiideaの実行が終了した際に呼び出されます。`blockName`は終了したPieceのタイプ(トリガー、サービス、アクションのいずれか)です。異常終了の場合、どのPieceまで到達したかが分かります。また、`error`にエラー情報が含まれます。

- `onLogin(applets:)`

  Riiiverへのログインが完了した際に呼び出されます。

- `onLogout()`

  Riiiverからログアウトした際に呼び出されます。

##### note:

- appletは、Riiiver開発時のiiideaの呼称です。コード内やドキュメント内にappletという表記が混在している箇所がありますが、iiideaと読み替えてください。

### 認証系機能(ログイン/ログアウト)を使用する

#### この節で説明するもの:

- [method] `getLoginUrl()`
- [method] `verifyCodeOf(uri:)`
- [method] `logout()`

Riiiverへログインするための認証ページを用意しています。`getLoginUrl()`メソッドの戻り値のURLを任意の方法で開いてください。
認証ページでユーザーがログイン操作を完了するとリダイレクトが発生します。リダイレクトURLは[アプリ情報の登録](https://developer.riiiver.com/docs/riiiversdk-usage/3546/)で設定した値です。カスタムスキーム経由でアプリが起動できるよう`info.plist`に設定してください。
カスタムスキーム経由でアプリが起動すると、URLのクエリパラメータとして短命の認可コードが付与されています。`AppDelegate`内の`application(_:open:options:)`デリゲートメソッド、または、SwiftUIを使用している場合は`onOpenURL(perform:)`メソッドで取得したURLを`verifyCodeOf(uri:)`メソッドに渡してRiiiverへのログインを完了してください。

##### note:

- リダイレクトを発生させカスタムスキームによるアプリへの値渡しが行われるため、認証ページを開く際は`SFSafariViewController`の利用を推奨します。`UIApplication.shared.open(url:)`メソッドを使用して外部ブラウザで開く場合、ユーザーがデフォルトのブラウザをSafariから変更している可能性があるためです。

Riiiverからログアウトしたい場合は、任意のタイミングで`logout()`メソッドを実行します。

### iiideaのリスト表示や設定画面を使用する

#### この節で説明するもの:

- [class] `ERUserWebPagesView`
- [protocol] `ERUserWebPagesViewDelegate`

ユーザーがiiideaを探したり入手したりするためのiiideaストアや、入手済みのiiideaの設定や削除を行うためのリストビューなどを集約したWebページを用意しています。このWebページを表示するためのクラスとして、`WKWebView`クラスを継承した`ERUserWebPagesView`クラスを用意しています。
UIViewControllerを使用する場合 `viewDidLoad()`メソッド内で、
SwiftUIのUIViewRepresentableを使用する場合 `makeUIView(context:)`メソッド内で、
`loadPage(type:)`メソッドを実行するとWebページを表示できます。

iiideaの中には、実行時に端末の権限を使用したり、サードパーティ製のサービスのOAuth認証が必要なものがあります。Webページ上でユーザーがそれらの権限を要求した際に、そのイベントの処理をアプリ側に委譲するための`ERUserWebPagesViewDelegate`プロトコルが用意されています。このdelegateには以下のようなdelegateメソッドが用意されていますので、必要な処理を実装してください。

##### `ERUserWebPagesViewDelegate`のメソッド

- `didReceivePermissionRequest(permissionType:)`

  Webページ上の端末権限の要求ボタンがタップされた際に呼ばれます。要求されている権限のタイプが引数で渡されるので、権限の利用許諾ダイアログをユーザーに提示してください。

- `didReceiveOpenUrlRequest(url:)`

  OAuth認証など新たにURLを開く必要がある場合に呼ばれます。引数で渡されるURLを開いてください。

- `didReceiveCloseRequest()`

  Webページ上の閉じるボタンがタップされた際に呼ばれます。

- `onError(error:)`

  Webページ上でエラーが発生した際に呼ばれます。

iiideaに必要な設定が完了すると、iiideaをアクティベートできるようになります。iiideaをアクティベートすると、実行待機状態になります。

### 事前にiiideaを入手済みの状態にする

#### この節で説明するもの:

- [method] `forcedDownloadApplet(appletIds:)`

ユーザーがアプリをインストールした時点で、いくつかの機能をプリインストール済みにしておきたいケースがあると思います。その場合は、`forcedDownloadApplet(appletIds:)`メソッドを使うことで、引数で指定したIDのiiideaを入手済みの状態にすることが可能です。

### 実行可能なiiideaの一覧を取得する

#### この節で説明するもの:

- [method] `getMyApplets(deviceIdList:)`
- [class] `ERApplet`

このサンプルアプリでは、Slate+の評価ボード上にある2つの物理ボタンを押した時に、それぞれ別のiiideaを実行できるように実装しています。そのため、実行するiiideaをユーザーが自由に選択できるようなUIをアプリ側で用意しています。この時、ユーザーが入手済みのiiidea一覧を取得するためのメソッドが`getMyApplets(deviceIdList:)`です。引数としてStringのArrayでデバイスのIDを渡すことで、そのデバイスで動作するiiideaの一覧を取得することができます。戻り値は`ERApplet`のArrayです。

`ERApplet`クラスには、iiideaのメタデータを含むプロパティの他、アクティベート・ディアクティベートやiiideaの実行に関するメソッドなどが用意されています。iiideaが実行可能な状態(アクティベート済みの状態)かどうかを知るためには、`getActivateStatus()`メソッドを実行してください。

### Pieceを実装してiiideaを実行する

#### この節で説明するもの:

- [protocol] `ERBlockExecutor`
- [protocol] `ERBlockDelegate`

iiideaは3つのPieceを組み合わせたスモールアプリケーションです。

アプリ・SDKに実装されるPieceは全て`ERBlockExecutor`プロトコルを継承する必要があります。このプロトコルには以下のメソッドが定義されていますので、必要な処理を実装してください。

##### `ERBlockExecutor`のメソッド

- `enable(applet:)`

  このPieceを使用する`applet`がユーザーによってアクティベートされる時に呼び出されます。アクティベート可能な場合はtrueを、アクティベートできない場合はfalseを返却してください。

  本サンプルアプリには実装例がありませんが、SDKに実装されているジオフェンストリガーでは、例えば、位置情報の利用許可が得られているかどうかを確認して、許可が得られていない場合はfalseを返却するように実装しています。

- `execute(applet:blockName:preferenceJson:inputJson:handler:)`

  このPieceを使用する`applet`が実行され、このPieceの実行順になった時に呼び出されます。Pieceの処理を完了するタイミングで、`handler`として渡される`ERBlockDelegate`の`completed(outputJson:error:)`メソッドを実行します。異常終了する場合も、引数`error`を渡して`completed(outputJson:error:)`メソッドを呼び出してください。

  iiideaの最初のPieceであるトリガーPieceの場合、アクティベートされた瞬間にこのメソッドが呼び出されることに注意してください。所望のイベントが実際に発生したタイミングでiiideaの実行が開始されるよう工夫が必要です。

  Slate+のEVB上のボタンがユーザーによって押されたタイミングで発火する`QCC5100TriggerPieceCore`では、iiideaIdとそのiiideaIdに対応する`ERBlockDelegate`をキーバリューペアとするDictionary型で`handler`を保持しています。そして、イベントを通知するための`didTrigger(appletId:input:)`メソッドを実装しています。`CBPeripheralDelegate`の`peripheral(_:didUpdateValueFor:error:)`メソッドが呼ばれるタイミング(すなわち、EVB上のボタン押しイベントがアプリに通知されるタイミング)で、`didTrigger(appletId:input:)`メソッドを呼び出してiiideaの実行を開始しています。

- `disable(applet:)`

  このPieceを使用する`applet`がディアクティベートされる時に呼び出されます。実行を待機している間、保持している必要があったリソースを解放するなどの処理を実装してください。

  `QCC5100TriggerPieceCore`では、iiideaIdに対応する`ERBlockDelegate`を破棄しています。また、SDKに実装されているジオフェンストリガーでは、位置情報の監視を停止しています。


### Firebaseを利用してアプリ外部からiiideaを実行する

#### この節で説明するもの:

- [method] `setNotificationToken(_:)`
- [method] `executeTaskForRemoteNotification(_:)`

##### 準備

この機能を実現するため、Firebase Cloud Messaging APIを使用しています。機能を利用するためには、以下のファイルが必要です。

- GoogleService-info.plist

- APNs authentication key


ファイルを準備する方法については以下のドキュメントを参照してください。

[Registering a key](https://firebase.flutter.dev/docs/messaging/apple-integration#1-registering-a-key)

[Registering an App Identifier](https://firebase.flutter.dev/docs/messaging/apple-integration#2-registering-an-app-identifier)

[Generating a provisioning profile](https://firebase.flutter.dev/docs/messaging/apple-integration#3-generating-a-provisioning-profile)

[Enable Push Notifications](https://firebase.flutter.dev/docs/messaging/apple-integration#enable-push-notifications)

[Enable Background Modes (iOS only)](https://firebase.flutter.dev/docs/messaging/apple-integration#enable-background-modes-ios-only)

用意した`Google-Service-info.plist`ファイルは、バンドルのルート階層に配置されている必要があります。本プロジェクトでは`Infra/Firebase/GoogleService`というディレクトリ内に配置してありますが、XcodeのRun Scriptでビルド実行時にバンドルのルートにファイルをコピーしています。

##### FirebaseMessagingのセットアップ

FirebaseMessagingの機能を利用するため、AppDelegateにいくつかの実装が必要です。AppDelegateの`application(_:didFinishLauchingWithOptions:)`メソッド内で、(1)FCMトークンの発行リクエスト、(2)FCMのデリゲートの設定、(3)プッシュ通知の権限要求、(4)APNsデバイストークンの発行リクエストをします。

```Swift
// (1) Request to issue FCM token and set delegates
FirebaseApp.configure()     
// (2) Set messaging delegate
Messaging.messaging().delegate = self
        
// Register for remote notifications.
// (3) This shows a permission dialog on first run, to show the dialog at
// a more appropriate time move this registration accordingly.
if #available(iOS 10.0, *) {
    // For iOS 10 display notification (sent via APNS)
    UNUserNotificationCenter.current().delegate = self

    let authOptions: UNAuthorizationOptions = [.alert, .badge, .sound]
    UNUserNotificationCenter.current().requestAuthorization(
          options: authOptions,
          completionHandler: { _, _ in }
    )
} else {
    let settings: UIUserNotificationSettings = UIUserNotificationSettings(types: [.alert, .badge, .sound], categories: nil)
    application.registerUserNotificationSettings(settings)
}

// (4) Request to issue APNs device token
application.registerForRemoteNotifications()
```

`application(_:didRegisterForRemoteNotificationsWithDeviceToken:)`メソッドを実装します。APNsのデバイストークンが発行されるとこのメソッドが呼び出されるので、発行されたデバイストークンをFirebase Messaging SDKにセットします。Firebase Messaging SDKにAPNsのデバイストークンが登録されるとFCMトークンが発行されます。

```Swift
func application(_ application: UIApplication,
                 didRegisterForRemoteNotificationsWithDeviceToken deviceToken: Data) {
    debugPrint("[APNs]デバイストークン受信 [\(deviceToken.map{ String(format: "%.2hhx", $0) }.joined())]")
    Messaging.messaging().apnsToken = deviceToken
}
```

`MessagingDelegate`プロトコルを継承し、`messaging(_:didReceiceRegistrationToken:)`メソッドを実装します。FCMトークンが発行されるとこのメソッドが呼び出されるので、FCMトークンをRiiiver SDKにセットします。`setNotificationToken(_:)`メソッドを実行してください。

```Swift
extension AppDelegate: MessagingDelegate {
    func messaging(_ messaging: Messaging, didReceiveRegistrationToken fcmToken: String?) {
        if let fcmToken {
            print("fcmToken = \(String(describing: fcmToken))")
            self.fcmToken = fcmToken
            Task(priority: .medium) {
                do {
                    try await RiiiverSDKClient.sharedInstance.setNotificationToken(fcmToken)
                } catch (let error) {
                    print("## MessagingDelegate setNotificationToken failed due to \(error.localizedDescription)")
                }
            }
        } else {
            debugPrint("fcm token is nil.")
            return
        }
    }
}
```

`application(_:didReceiveRemoteNotification:fetchCompletionHandler:)`メソッドを実装します。Firebaseを利用したデータメッセージを受信するとこのメソッドが呼ばれます([cf. メッセージのタイプ](https://firebase.google.com/docs/cloud-messaging/concept-options?hl=ja#notifications_and_data_messages))。受信したプッシュ通知をRiiiver SDKに渡して、後続処理を委譲します。`executeTaskForRemoteNotification(_:)`メソッドを使用してください。

Riiiverを利用したデータメッセージの場合、`userInfo`の中に`"notification_id"`というキーでプッシュ通知の分類情報が含まれています。あなたのプロジェクトがすでにFirebaseを利用している場合は、このキーの有無によって処理を分岐させることができます。すなわち、このキーが含まれているかどうかを判定して、含まれている場合のみ、`executeTaskForRemoteNotification(_:)`メソッドを実行するようにしてください。

## アプリのアーキテクチャ

[Clean Architecture](https://g.co/kgs/d4EKLj)のSwift版こと[VIPERアーキテクチャ](https://cheesecakelabs.com/blog/ios-project-architecture-using-viper/)をベースに、Swift UIの状態管理手法を用いている。

外部からデータを取得してきて表示するようなケースにおいて、従来のUIKitベースのVIPERでは、InteractorからPresenterへ通知→PresenterからViewへ更新依頼というイベントの流れで実現されているが、Swift UIの採用に際して、EntityをObservable ObjectとしViewからの監視対象とし、値が変更された際に再描画するようにしている。

cf: VIPERアーキテクチャについて、[@hicka04](https://github.com/hicka04)氏による日本語での[解説記事](https://qiita.com/hicka04/items/09534b5daffec33b2bec)が参考になった。



## Tips: 接続先の切り替え方について(not include in public-release)

XcodeのBuild Configurationを用いて各種RiiiverのリソースのベースURLやベンダーID/デバイスIDのような環境によって異なる定数などを切り替えられるようにしている。

![img](./readme_rsc/select_scheme.png "select_scheme")

- バンドルID, 表示名, ログインスキーマ

  - .xcconfig内で定義

- erconfiguration.json

  - Configurationの値に応じて該当するファイルをアプリのembedded resource内にコピーするスクリプトを追加することで対応。

  - [TARGETS] → [Build Phases]内にあるcopy erconfigurationという名称のシェルスクリプト。

    ![img](./readme_rsc/script_cper.png "script")

- デバイスID, 強制DL対象のiiideaID, OAuth連携用のベースURL

  - Configurationに応じてカスタムフラグの値を変更し、コード内でフラグの値を確認して切り替えている。

  - カスタムフラグの編集は[TARGETS] → [BuildSettings] → [Swift Compiler - Custom Flags] の値を変更する。

    ![img](./readme_rsc/custom_flag.png "custom_flag")

