//
//  AppDelegate.swift
//  qcc5100.sample
//

import UIKit
import RiiiverSDK
import FirebaseCore
import FirebaseMessaging

class AppDelegate: NSObject, UIApplicationDelegate {
    var fcmToken: String?
    
    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) -> Bool {
        // 通知機能を使うPieceがあるため必ず設定する
        UNUserNotificationCenter.current().delegate = self
        
        // For IoT Device Link: Request to issue FCM token and set delegates
        FirebaseApp.configure()
        
        // [START set_messaging_delegate]
        Messaging.messaging().delegate = self
        // [END set_messaging_delegate]
        
        // Register for remote notifications. This shows a permission dialog on first run, to
        // show the dialog at a more appropriate time move this registration accordingly.
        // [START register_for_notifications]
        if #available(iOS 10.0, *) {
            // For iOS 10 display notification (sent via APNS)
            UNUserNotificationCenter.current().delegate = self

            let authOptions: UNAuthorizationOptions = [.alert, .badge, .sound]
            UNUserNotificationCenter.current().requestAuthorization(
                options: authOptions,
                completionHandler: { _, _ in }
            )
        } else {
            let settings: UIUserNotificationSettings = UIUserNotificationSettings(types: [.alert, .badge, .sound], categories: nil)
            application.registerUserNotificationSettings(settings)
        }

        application.registerForRemoteNotifications()

        // FIXME: comment out when it is available to use device-link w/ SDK2.0
//        ERSDK.sharedInstance.deviceLink = self
        
        return true
    }

    func application(_ app: UIApplication, open url: URL, options: [UIApplication.OpenURLOptionsKey : Any] = [:]) -> Bool {
        return true
    }

    func applicationWillResignActive(_ application: UIApplication) {
        // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
        // Use this method to pause ongoing tasks, disable timers, and invalidate graphics rendering callbacks. Games should use this method to pause the game.
    }

    func applicationDidEnterBackground(_ application: UIApplication) {
        // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
        // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
    }

    func applicationWillEnterForeground(_ application: UIApplication) {
        // Called as part of the transition from the background to the active state; here you can undo many of the changes made on entering the background.
    }

    func applicationDidBecomeActive(_ application: UIApplication) {
        // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
    }

    func applicationWillTerminate(_ application: UIApplication) {
        // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
    }

    // MARK: - For Remote Notification (APNs)
    func application(_ application: UIApplication,
                     didRegisterForRemoteNotificationsWithDeviceToken deviceToken: Data) {
        debugPrint("[APNs]デバイストークン受信 [\(deviceToken.map{ String(format: "%.2hhx", $0) }.joined())]")
        Messaging.messaging().apnsToken = deviceToken
    }
    
    func application(_ application: UIApplication,
                     didFailToRegisterForRemoteNotificationsWithError error: Error) {
        debugPrint("[APNs] プッシュ通知が利用できません。[\(error.localizedDescription)]")
    }

    func application(_ application: UIApplication,
                     didReceiveRemoteNotification userInfo: [AnyHashable : Any],
                     fetchCompletionHandler completionHandler: @escaping (UIBackgroundFetchResult) -> Void) {
        debugPrint("[APNs] プッシュ通知を受信しました。[\(userInfo.debugDescription)]")
        // For IoT Device Link: Pass through the remote notification to SDK
        Task(priority: .high) {
            if let notificationId = userInfo["notification_id"] as? String,
               notificationId == "link" {
                // Remote Notification for iiidea execution
                debugPrint("[APNs] iiidea実行用のプッシュ通知です。iiideaを実行します。")
                guard BLECentralClient.sharedInstance.entity.phoneToWatchQue == nil else { return }
                BLECentralClient.sharedInstance.entity.phoneToWatchQue = CommandReceiverCommand.display
            }
            try await RiiiverSDKClient.sharedInstance.executeTaskForRemoteNotification(userInfo)
        }
        // FIXME: タイミングここで大丈夫か？
        completionHandler(UIBackgroundFetchResult.newData)
    }
}


extension AppDelegate: UNUserNotificationCenterDelegate {
    // Receive displayed notifications for iOS 10 devices.
    func userNotificationCenter(_ center: UNUserNotificationCenter,
                                willPresent notification: UNNotification) async -> UNNotificationPresentationOptions {
        let userInfo = notification.request.content.userInfo
        
        // With swizzling disabled you must let Messaging know about the message, for Analytics
         Messaging.messaging().appDidReceiveMessage(userInfo)

        // Print full message.
        print(userInfo)

        // Change this to your preferred presentation option
        // 通知機能を使うPieceがあるため必ず設定する
        if #available(iOS 14.0, *) {
            return[[.banner, .list, .sound]]
        } else {
            return[[.alert, .sound]]
        }
    }

    func userNotificationCenter(_ center: UNUserNotificationCenter,
                                didReceive response: UNNotificationResponse) async {
        let userInfo = response.notification.request.content.userInfo
        // With swizzling disabled you must let Messaging know about the message, for Analytics
         Messaging.messaging().appDidReceiveMessage(userInfo)

        // Print full message.
        print(userInfo)
        
//        guard let vc = UIApplication.topViewController() else {
//            return
//        }
        // TODO: For IoT Device Link: Open a dialog that requests users to allow using device-linked-iiidea and/or canceling link.
//        ERSDK.sharedInstance.openDialog(vc, openArg: response.notification.request.content.userInfo)
  }
}

// MARK: - For FirebaseMessaging
extension AppDelegate: MessagingDelegate {
    func messaging(_ messaging: Messaging, didReceiveRegistrationToken fcmToken: String?) {
        if let fcmToken {
            print("fcmToken = \(String(describing: fcmToken))")
            self.fcmToken = fcmToken
            Task(priority: .medium) {
                do {
                    try await RiiiverSDKClient.sharedInstance.setNotificationToken(fcmToken)
                } catch (let error) {
                    print("## MessagingDelegate setNotificationToken failed due to \(error.localizedDescription)")
                }
            }
        } else {
            debugPrint("fcm token is nil.")
            return
        }
    }
}

// MARK: - For RiiiverSDK2.0
// FIXME: comment out when it is available to use friend-link w/ SDK3.0?
//extension AppDelegate: ERDeviceLinkDelegate {
//    func requestOpenAcceptFriendPage(url: String) {
//        // TODO: open accept page
//    }
//
//    func requestOpenPreference(appletId: String, userPreferenceId: String) {
//        // TODO: open preference view
////        if let currentVC = UIApplication.topViewController() {
////            let preferenceVC = UIStoryboard(name: "Main", bundle: nil).instantiateViewController(withIdentifier: "PreferenceWebVC") as! PreferenceWebViewController
////            preferenceVC.identifier = appletId
////            currentVC.present(preferenceVC, animated: true, completion: nil)
////        }
//    }
//}
