//
//  AppNavigatePath.swift
//  qcc5100.sample
//

import Foundation

public enum ModulePath: Equatable {
    case riiiverLogin
    case userWebView
    case deviceSearch
    case deviceSettings
}

class AppNavigatePath: ObservableObject {
    @Published var current: ModulePath
    @Published var pathHistory: [ModulePath]
    
    private init() {
        self._current = Published(initialValue: .deviceSearch)
        self._pathHistory = Published(initialValue: [.deviceSearch])
    }
    
    static let sharedInstance: AppNavigatePath = AppNavigatePath()

    func push(to nextPath: ModulePath) {
        DispatchQueue.main.async { [self] in
            pathHistory.append(nextPath)
            current = nextPath
        }
    }
    
    func pop() {
        guard canPop() else { return }
        DispatchQueue.main.async { [self] in
            pathHistory.removeLast()
            current = pathHistory.last!
        }
    }
    
    func canPop() -> Bool {
        return pathHistory.count >= 2
    }
    
    func popTo(_ priorPath: ModulePath) {
        guard let index = pathHistory.lastIndex(of: priorPath),
              index < pathHistory.count else {
            return
        }
        DispatchQueue.main.async { [self] in
            pathHistory.removeLast(pathHistory.count - (index + 1))
            current = pathHistory.last!
        }
    }
    
    func popToTop() {
        DispatchQueue.main.async { [self] in
            pathHistory.removeLast(pathHistory.count - 1)
            current = pathHistory.last!
        }
    }    
}
