//
//  Data+QCC5100Sample.swift
//  qcc5100.sample
//

import Foundation

/// Data Extension
extension Data {
    /**
     * - Returns: String in hex
     */
    var hexString: String {
        return self.map{ String(format: "%02X", $0) }.joined()
    }
    
    /// 指定インデックスから指定の長さだけ切り出したデータを返す。
    /// - Parameters:
    ///   - index: 開始インデックス
    ///   - length: 切り出す長さ
    /// - Returns: データ
    func subdata(of index: Int, length: Int) -> [UInt8] {
        let s = index
        var e = index + length
        if e > self.count {
            e = self.count
        }
        return subdata(in: s..<e).map{ UInt8($0) }
    }
    
    /// Dataを任意の型の数値に変換する
    /// - Parameters:
    ///   - type: 型を指定する
    /// - Returns: 指定した型の数値
    func to<T>(type: T.Type) -> T {
        return withUnsafeBytes {$0.load(as: type)}
    }
    
    /// Date型に変換する
    var dateTime: Date? {
        // データ長が不正な場合即リターンでnilを返却
        guard self.count == 7 else { return nil }
        
        let year = Data(self[0...1]).to(type: Int16.self)
        let month = Int(self[2])
        let day = Int(self[3])
        let hour = Int(self[4])
        let minute = Int(self[5])
        let second = Int(self[6])
        let dateComponent = DateComponents(
            calendar: Calendar(identifier: .gregorian),
            year: Int(year),
            month: month,
            day: day,
            hour: hour,
            minute: minute,
            second: second)
        return dateComponent.date
    }
    
    var slateRiiiverFormat: SlateRiiiverPayload {
        return SlateRiiiverPayload(
            dataLength: self.subdata(of: 0, length: 2),
            commandId: self[2],
            message: self.dropFirst(3).map { UInt8($0) }
        )
    }
}

struct SlateRiiiverPayload {
    let dataLength: [UInt8]
    let commandId: UInt8
    let message: [UInt8]
}

extension SlateRiiiverPayload {
    var combinedByteArray: [UInt8] {
        return self.dataLength + [self.commandId] + self.message
    }
    
    var combinedData: Data {
        return Data(self.combinedByteArray)
    }
}
