//
//  QCC5100ActionPieceCore.swift
//  qcc5100.sample
//

import RiiiverSDK

@objc(QCC5100ActionPieceCore)
class QCC5100ActionPieceCore: NSObject {
    var identifier: String?
    var output: [String: Any]?
    var handler: RiiiverSDK.ERBlockDelegate?
    let semaphore = DispatchSemaphore(value: 0)
    
    @objc private func didReceiveActionResponse(_ notification: Notification) {
        guard let userInfo = notification.userInfo else { return }
        
        // Stop observation of app's internal notification
        NotificationCenter.default.removeObserver(self,
                                                  name: BlockNotification.Name.ActionResponse,
                                                  object: nil)
        
        var error: RiiiverSDK.ERError?

        // 結果がfalseの場合はエラーを設定する
        if let success = userInfo[BlockNotification.Key.Result] as? Bool {
            if !success {
                print("## QCC5100ActionPieceCore: error has occured.)")
                error = ERError(ERErrorCode.someError)
            }
        }
                
        if let error {
            print("## QCC5100ActionPieceCore: ended. output = \(output ?? [:]), error:\(error.errorMessage)")
        } else {
            print("## QCC5100ActionPieceCore: ended. output = \(output ?? [:])")
        }
        
        handler?.completed(outputJson: output, error: error)
        semaphore.signal()
    }
}

extension QCC5100ActionPieceCore: ERBlockExecutor {
    func enable(applet: RiiiverSDK.ERApplet) -> Bool {
        return true
    }
    
    func disable(applet: RiiiverSDK.ERApplet) {
        //
    }
    
    func execute(applet: RiiiverSDK.ERApplet,
                 blockName: RiiiverSDK.ERBlockName,
                 preferenceJson: Dictionary<String, Any>?,
                 inputJson: Dictionary<String, Any>?,
                 handler: RiiiverSDK.ERBlockDelegate
    ) {
        self.identifier = applet.getId()
        self.handler = handler
        
        print("## QCC5100ActionPieceCore: execute - appletId = \(applet.getId())")
        
        guard let input = inputJson?["inputText"] as? String else {
            print("## QCC5100ActionPieceCore: input missing")
            handler.completed(outputJson: nil, error: ERError(ERErrorCode.invalidBlock))
            return
        }
        
        var outputString = input
        if let preText = preferenceJson?["preText"] as? String,
           preText.count != 0 {
            outputString = preText + outputString
        }
        if let postText = preferenceJson?["postText"] as? String,
           postText.count != 0 {
            outputString = outputString + postText
        }
        output = ["output": outputString]
        
        // Check BLE Queue
        guard let queue = BLECentralClient.sharedInstance.entity.phoneToWatchQue else {
            print("## QCC5100ActionPieceCore: queue missing")
            handler.completed(outputJson: output, error: ERError(ERErrorCode.someError))
            return
        }

        // Generate BLERequest
        var request: BLEOperation
        switch queue {
        case .button1result:
            request = BLEAPI.SendResultButton1(message: outputString)
        case .button2result:
            request = BLEAPI.SendResultButton2(message: outputString)
        case .display:
            request = BLEAPI.SendDisplayData(message: outputString)
        case .success: fallthrough
        case .failure: fallthrough
        default:
            print("## QCC5100ActionPieceCore: unhandlable queue found")
            handler.completed(outputJson: output, error: ERError(ERErrorCode.someError))
            return
        }
        
        // Start to observe app's internal notification
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(didReceiveActionResponse(_:)),
                                               name: BlockNotification.Name.ActionResponse,
                                               object: nil)
        
        // Invoke BLERequest
        BLECentralClient.sharedInstance.operate(request: request)
        
        // Wait for response from QCC5100
        DispatchQueue.global(qos: .default).async {
            self.semaphore.wait()
        }
    }
}
