//
//  QCC5100TriggerPieceCore.swift
//  qcc5100.sample
//

import RiiiverSDK

@objc(QCC5100TriggerPieceCore)
class QCC5100TriggerPieceCore: NSObject {
    static var delegates: Dictionary<String, ERBlockDelegate> = [:]
    
    // QCC5100TriggerPieceCore を実行したいタイミングでこのメソッドを実行する.
    // Invoke when you would like to kick the QCC5100TriggerPieceCore.
    static func didTrigger(appletId: String, input: Dictionary<String, Any>?) {
        guard !self.delegates.isEmpty,
              self.delegates.keys.contains(where: { $0 == appletId }) else {
            // 実行したい対象のERAppletを実行するためのhandlerを保持していないため実行不可
            // Cancelled as handler(ERBlockDelegate) for execute ERApplet identified by [appletId] does not present.
            return
        }
        
        // Triggerのアウトプットを生成
        // - Get the text field's string and set it as a output of the T piece.
        var output = [ "triggerSampleInfo" : "" ]
        if let input {
            output["triggerSampleInfo"] = input[BlockNotification.Key.Output] as? String
        }
        
        print("## QCC5100TriggerPieceCore Trigger is fired! output:\(String(describing: output))")
        
        // Trigger with outpur information - output情報を設定してTrigger発火
        self.delegates[appletId]?.completed(outputJson: output, error: nil)
    }
}


extension QCC5100TriggerPieceCore: ERBlockExecutor {    
    func enable(applet: RiiiverSDK.ERApplet) -> Bool {
        print("## QCC5100TriggerPieceCore: enable is invoked(ACTIVATE). \(applet.metaData.getTitle())")
        return true
    }
    
    func execute(applet: ERApplet,
                 blockName: ERBlockName,
                 preferenceJson: Dictionary<String, Any>?,
                 inputJson: Dictionary<String, Any>?,
                 handler: ERBlockDelegate
    ) {
        print("## QCC5100TriggerPieceCore: execute is invoked (ACTIVATED). \(applet.metaData.getTitle())")

        let group = DispatchGroup()
        group.enter()
        DispatchQueue.main.async {
            QCC5100TriggerPieceCore.delegates.updateValue(handler, forKey: applet.getId())
            group.leave()
        }
    }
    
    func disable(applet: RiiiverSDK.ERApplet) {
        print("## QCC5100TriggerPieceCore: disable is invoked (DEACTIVATED). \(applet.metaData.getTitle())")

        let group = DispatchGroup()
        group.enter()
        DispatchQueue.main.async {
            QCC5100TriggerPieceCore.delegates.removeValue(forKey: applet.getId())
            group.leave()
        }
    }
}

extension QCC5100TriggerPieceCore {
    static var className: String {
        return String(describing: self) // ClassName
    }
}
