//
//  TriggerPieceCoreSample.swift
//  qcc5100.sample
//

import RiiiverSDK

@objc(TriggerPieceCoreSample)
class TriggerPieceCoreSample: NSObject, ERBlockExecutor {
    static var delegates: Dictionary<String, ERBlockDelegate> = [:]
    
    // TriggerPieceCoreSample を実行したいタイミングでこのメソッドを実行する.
    // Invoke when you would like to kick the TriggerPieceCoreSample.
    static func didTrigger(appletId: String, input: Dictionary<String, Any>?) {
        guard !self.delegates.isEmpty,
              self.delegates.keys.contains(where: { $0 == appletId }) else {
            // 実行したい対象のERAppletを実行するためのhandlerを保持していないため実行不可
            // Cancelled as handler(ERBlockDelegate) for execute ERApplet identified by [appletId] does not present.
            return
        }
        
        // TriggerPieceCoreSampleのアウトプットを生成
        // - Get the text field's string and set it as a output of the T piece.
        var output = [ "triggerSampleInfo" : "" ]
        if let input {
            output["triggerSampleInfo"] = input[BlockNotification.Key.Output] as? String
        }
        
        print("## TriggerPieceCoreSample Trigger is fired! output:\(String(describing: output))")
        
        // Trigger with outpur information - output情報を設定してTrigger発火
        self.delegates[appletId]?.completed(outputJson: output, error: nil)
    }

    
    func enable(applet: RiiiverSDK.ERApplet) -> Bool {
        print("## TriggerPieceCoreSample: enable invoked (ACTIVATE). \(applet.metaData.getTitle())")
        return true
    }
    
    func execute(
        applet: RiiiverSDK.ERApplet,
        blockName: RiiiverSDK.ERBlockName,
        preferenceJson: Dictionary<String, Any>?,
        inputJson: Dictionary<String, Any>?,
        handler: RiiiverSDK.ERBlockDelegate)
    {
        print("## TriggerPieceCoreSample: execute invoked (ACTIVATED). \(applet.metaData.getTitle())")
        TriggerPieceCoreSample.delegates.updateValue(handler, forKey: applet.getId())
    }
    
    func disable(applet: RiiiverSDK.ERApplet) {
        print("## TriggerPieceCoreSample: disable invoked (DEACTIVATED). \(applet.metaData.getTitle())")
        TriggerPieceCoreSample.delegates.removeValue(forKey: applet.getId())
        return
    }
}

extension TriggerPieceCoreSample {
    static var className: String {
        return String(describing: self) // ClassName
    }
}
