//
//  GetAppInfoInteractor.swift
//  qcc5100.sample
//

import Foundation

typealias AppInfo = (name: String, build: String, version: String)

protocol GetAppInfoUsecase: AnyObject {
    func getInfo() -> AppInfo
}

final class GetAppInfoInteractor {
    private var name: String = ""
    private var build: String = ""
    private var version: String = ""

    init() {
        guard let name = Bundle.main.object(forInfoDictionaryKey: "CFBundleName") as? String,
              let build = Bundle.main.object(forInfoDictionaryKey: "CFBundleVersion") as? String,
              let version = Bundle.main.object(forInfoDictionaryKey: "CFBundleShortVersionString") as? String
        else {
            return
        }
        self.name = name
        self.build = build
        self.version = version
    }
}

extension GetAppInfoInteractor: GetAppInfoUsecase {
    func getInfo() -> AppInfo {
        return AppInfo(name: name, build: build, version: version)
    }
}
