//
//  ManageIiideaInteractor.swift
//  qcc5100.sample
//

import Foundation
import RiiiverSDK

protocol ManageIiideaUsecase: AnyObject {
    func getMyApplets() async throws -> Void
    func removeMyApplet(_ iiideaId: String) async throws -> Void
    func toggleActivateState(of iiideaId: String, newValue: Bool) async -> (unauthorizedPermissionList: [ERPermissionType]?, unauthorizedProxyList: [String]?, error: Error?)
    func kickTriggerPieceCoreSample(iiideaId: String, triggerInput: String) async -> Void
    func changeTargetIiidea(of: CommandReceiverCommand, iiideaId: String?)
    func clearMyApplets()
}

final class ManageIiideaInteractor {
    private var riiiverSdkClient: RiiiverSDKClientInterface
    
    init(riiiverSdkClient: RiiiverSDKClientInterface = RiiiverSDKClient.sharedInstance) {
        self.riiiverSdkClient = riiiverSdkClient
    }
}

extension ManageIiideaInteractor: ManageIiideaUsecase {
    func getMyApplets() async throws -> Void {
        try await riiiverSdkClient.getMyApplets()
    }
    
    func removeMyApplet(_ iiideaId: String) async throws -> Void {
        try await riiiverSdkClient.removeMyApplet(iiideaId)
    }
    
    func toggleActivateState(of iiideaId: String, newValue: Bool) async -> (unauthorizedPermissionList: [ERPermissionType]?, unauthorizedProxyList: [String]?, error: Error?) {
        guard let iiidea = riiiverSdkClient.getERAppletInstance(of: iiideaId) else {
            return (nil, nil, "ERAppletインスタンスの取得に失敗")
        }
        let (isActivated, gase) = await iiidea.getActivateStatus()
        guard gase == nil else {
            return (nil, nil, "アクティベート状態の取得に失敗")
        }
        if isActivated == newValue {
            // 既に所望のアクティベート状態なので何もしない
            return (nil, nil, nil)
        }
        let error = await iiidea.setActivateStatus(status: !isActivated)
        guard error == nil else {
            print("## toggle iiidea activate status failed due to: \(error!)")
            return (nil, nil, "アクティベート状態の変更に失敗")
        }
        return (nil, nil, nil)
    }
    
    func kickTriggerPieceCoreSample(iiideaId: String, triggerInput: String) async -> Void {
        guard let iiidea = riiiverSdkClient.getERAppletInstance(of: iiideaId) else {
            print("kick iiidea failed: No iiidea")
            return
        }
        
        let (isActivated, error) = await iiidea.getActivateStatus()
        guard isActivated == true, error == nil else {
            print("kick iiidea failed: Not Activated")
            return
        }
        
//        print("preference:\(iiidea.triggerBlockUserPref ?? [:])")
        
        var input: [String:Any] = [:]
        input[BlockNotification.Key.Output] = triggerInput
        TriggerPieceCoreSample.didTrigger(appletId: iiidea.getId(), input: input)
    }
    
    func changeTargetIiidea(of: CommandReceiverCommand, iiideaId: String?) {
        riiiverSdkClient.changeTargetIiidea(of: of, iiideaId: iiideaId)
    }
    
    func clearMyApplets() {
        riiiverSdkClient.clearMyApplets()
    }
}
