//
//  RequestPermissionInteractor.swift
//  qcc5100.sample
//

import Foundation
import RiiiverSDK

protocol RequestPermissionUsecase: AnyObject {
    func requestOsPermission(_ permissionType: String)
}

final class RequestPermissionInteractor {
    private var riiiverSdkClient: RiiiverSDKClientInterface
    private var permissionManager: PermissionManagerInterface
    
    init(
        riiiverSdkClient: RiiiverSDKClientInterface = RiiiverSDKClient.sharedInstance,
        permissionManager: PermissionManagerInterface = PermissionManager.sharedInstance
    ) {
        self.riiiverSdkClient = riiiverSdkClient
        self.permissionManager = permissionManager
    }
}

extension RequestPermissionInteractor: RequestPermissionUsecase {
    func requestOsPermission(_ permissionType: String) {
        print("## [RequestPermissionInteractor] requestOsPermission: in \(permissionType)")
        switch permissionType {
        case ERPermissionType.auth.rawValue:
            break
        case ERPermissionType.location.rawValue:
            self.permissionManager.request(request: PermissionRequest.Location())
            break
        case ERPermissionType.microphone.rawValue:
            self.permissionManager.request(request: PermissionRequest.Microphone())
            break
        case ERPermissionType.calendar.rawValue:
            self.permissionManager.request(request: PermissionRequest.Calendar())
            break
        case ERPermissionType.mediaLibrary.rawValue:
            self.permissionManager.request(request: PermissionRequest.MediaLibrary())
            break
        case ERPermissionType.notification.rawValue:
            self.permissionManager.request(request: PermissionRequest.Notification())
            break
        default:
            print("## [RequestPermissionInteractor] requestOsPermission: invalid or unknown permission type")
            break
        }
    }
}
