//
//  RequestPermissionInteractorTests.swift
//  qcc5100.sampleTests
//

import XCTest
import RiiiverSDK
@testable import qcc5100_sample

class RequestPermissionInteractorTests: XCTestCase {
    
    private var sdkClient: SDKClientMock!
    private var permissionManager: PermissionManagerMock!
    private var interactor: RequestPermissionInteractor!
    
    override func setUp() {
        self.sdkClient = .init()
        self.permissionManager = .init()
        self.interactor = .init(riiiverSdkClient: sdkClient, permissionManager: permissionManager)
    }
    
    override func tearDown() {
    }
    
    func test_requestOsPermission() {
        XCTContext.runActivity(named: "requestOsPermission") { _ in
            XCTContext.runActivity(named: "before called") { _ in
                XCTContext.runActivity(named: "permissionManager: `request` is not called") { _ in
                    XCTAssertEqual(permissionManager.callCount_request, 0)
                }
            }
            
            XCTContext.runActivity(named: "after called with valid permission type") { _ in
                XCTContext.runActivity(named: "permissionManager: `request` is called") { _ in
                    interactor.requestOsPermission("location")
                    XCTAssertEqual(permissionManager.callCount_request, 1)
                }
            }

            XCTContext.runActivity(named: "after called with invalid permission type") { _ in
                XCTContext.runActivity(named: "permissionManager: `request` is not called") { _ in
                    setUp()
                    interactor.requestOsPermission("unknown_permission")
                    XCTAssertEqual(permissionManager.callCount_request, 0)
                }
            }
        }
    }
    
    // MARK: - Mock
    class PermissionManagerMock: PermissionManagerInterface {
        var callCount_request = 0
        func request<Request>(request: Request) where Request : qcc5100_sample.IPermissionRequest {
            callCount_request += 1
        }
    }
    
    class SDKClientMock: RiiiverSDKClientInterface {
        init(emulateError: Bool = false) {
            self.isError_generateOAuthURL = emulateError
            self.isError_revokeOAuth = emulateError
        }

        func getMyApplets() async throws {
            // Not invoked from RequestPermissionInteractor
        }
        
        func removeMyApplet(_ iiideaId: String) async throws {
            // Not invoked from RequestPermissionInteractor
        }
        
        func getERAppletInstance(of iiideaId: String) -> RiiiverSDK.ERApplet? {
            // Not invoked from RequestPermissionInteractor
            return nil
        }

        var isError_generateOAuthURL = false
        var callCount_generateOAuthURL = 0
        func generateOAuthURL(blockId: String) async throws -> Void {
            callCount_generateOAuthURL += 1
            if isError_generateOAuthURL {
                throw ERError(ERErrorCode.noServerUrl)
            } else {
                return
            }
        }
        
        var isError_revokeOAuth = false
        var callCount_revokeOAuth = 0
        func revokeOAuth(of blockId: String) async throws {
            callCount_revokeOAuth += 1
            if isError_revokeOAuth {
                throw ERError.init(ERErrorCode.someError)
            }
        }
        
        func changeTargetIiidea(of: qcc5100_sample.CommandReceiverCommand, iiideaId: String?) {
            // Not invoked from RequestPermissionInteractor
        }
        
        func clearMyApplets() {
            // Not invoked from RequestPermissionInteractor
        }
        
        func setNotificationToken(_ fcmToken: String) async throws {
            //
        }
        
        func executeTaskForRemoteNotification(_ remoteNotification: [AnyHashable : Any]) async throws {
            //
        }
    }
}
