//
//  SearchDeviceInteractor.swift
//  qcc5100.sample
//

import Foundation
import CoreBluetooth

protocol SearchDeviceUsecase: AnyObject {
    func startScan()
    func connect(to peripheral: PeripheralEntity, completion: @escaping ((_ isSlateRiiiver: Bool) -> Void) )
    func disconnectPeripheral()
    func deleteFoundPeripherals()
}

final class SearchDeviceInteractor {
    private var bleCentralClient: BLECentralRequestable
    private var completion: ((_ :Bool) -> Void)?
    
    init(bleCentralClient: BLECentralRequestable = BLECentralClient.sharedInstance,
         completion: ( (_: Bool) -> Void)? = nil) {
        self.bleCentralClient = bleCentralClient
        self.completion = completion
    }
}

extension SearchDeviceInteractor: SearchDeviceUsecase {
    
    func startScan() {
        print("[SearchDeviceInteractor] didTapScanButotn: in")
        bleCentralClient.operate(request: BLEAPI.ToggleScanState())
    }
    
    func connect(to peripheral: PeripheralEntity, completion: @escaping ((_ isSlateRiiiver: Bool) -> Void)) {
        guard let rawInstance = peripheral.rawInstance else { return }
        print("[SearchDeviceInteractor] didTapPeripheral: in. name = \(rawInstance.name ?? "NO_NAME"), uuid = \(rawInstance.identifier.uuidString)")
        self.completion = completion
        let request = BLEAPI.Connect(uuid: rawInstance.identifier.uuidString, delegate: self)
        bleCentralClient.operate(request: request)
    }
    
    func disconnectPeripheral() {
        self.completion = nil
        bleCentralClient.operate(request: BLEAPI.CancelConnect())
    }
    
    func deleteFoundPeripherals() {
        bleCentralClient.operate(request: BLEAPI.StopScan())
        bleCentralClient.clearAllFounded()
    }
}

extension SearchDeviceInteractor: BLECentralDelegate {
    var identifier: String {
        return String(describing: self)
    }
    
    func didConnectPeripheral(_ peripheral: CBPeripheral) {
        print("[SearchDeviceInteractor] didConnectPeripheral: in. name = \(peripheral.name ?? "NO_NAME"), uuid = \(peripheral.identifier.uuidString)")
        bleCentralClient.operate(request: BLEAPI.StopScan())
        bleCentralClient.operate(request: BLEAPI.DiscoverService(delegate: self))
    }
    
    func didNotFoundService() {
        if let completion {
            completion(false)
        }
    }
    
    func didFoundService() {
        if let completion {
            completion(true)
        }
    }
}
