//
//  DeviceSearchResultPresenter.swift
//  qcc5100.sample
//

import Foundation

protocol DeviceSearchResultPresentation: AnyObject {
    func didTapScan()
    func didSelect(peripheral: PeripheralEntity)
    func didTapInfo()
    func didTapDelete()
}

final class DeviceSearchResultPresenter {
    private let viewDelegate: DeviceSearchResultViewInterface
    private let router: DeviceSearchResultWireframe
    private let searchDeviceInteractor: SearchDeviceUsecase
    private let getAppInfoInteractor: GetAppInfoUsecase
    
    init(viewDelegate: DeviceSearchResultViewInterface,
         router: DeviceSearchResultWireframe,
         searchDeviceInteractor: SearchDeviceUsecase,
         getAppInfoInteractor: GetAppInfoUsecase
    ) {
        self.viewDelegate = viewDelegate
        self.router = router
        self.searchDeviceInteractor = searchDeviceInteractor
        self.getAppInfoInteractor = getAppInfoInteractor
    }
}

extension DeviceSearchResultPresenter: DeviceSearchResultPresentation {
    func didTapScan() {
        searchDeviceInteractor.startScan()
    }
    
    func didSelect(peripheral: PeripheralEntity) {
        viewDelegate.toggleOverlay()
        searchDeviceInteractor.connect(to: peripheral) { [self] isSlateRiiiver in
            viewDelegate.toggleOverlay()
            if isSlateRiiiver {
                router.segueToSettings()
            } else {
                searchDeviceInteractor.disconnectPeripheral()
                viewDelegate.showAlertUnavailable()
            }
        }
    }
    
    func didTapInfo() {
        let info = getAppInfoInteractor.getInfo()
        viewDelegate.showAppInfo(info)
    }
    
    func didTapDelete() {
        searchDeviceInteractor.deleteFoundPeripherals()
    }
}
