//
//  DeviceSearchResultView.swift
//  qcc5100.sample
//

import SwiftUI

protocol DeviceSearchResultViewInterface {
    func toggleOverlay()
    func showAlertUnavailable()
    func showAppInfo(_ info: AppInfo)
}

final class DeviceSearchResultViewDelegate: ObservableObject, DeviceSearchResultViewInterface {
    @Published var isShowOverlay: Bool = false
    @Published var isUnavailable: Bool = false
    @Published var isShowAppInfo: Bool = false
    @Published var appInfo: AppInfo = AppInfo(name: "", build: "", version: "")
    
    func toggleOverlay() {
        self.isShowOverlay.toggle()
    }
    
    func showAlertUnavailable() {
        self.isUnavailable.toggle()
    }
    
    func showAppInfo(_ info: AppInfo) {
        self.appInfo = info
        self.isShowAppInfo.toggle()
    }
}

struct DeviceSearchResultView: View {
    @ObservedObject var dataSource: BLEEntity
    @StateObject private var delegate: DeviceSearchResultViewDelegate

    var presenter: DeviceSearchResultPresentation!

    init(delegate: DeviceSearchResultViewDelegate) {
        self._delegate = StateObject(wrappedValue: delegate)
        self.dataSource = BLECentralClient.sharedInstance.entity
    }
    
    var body: some View {
        NavigationView {
            ZStack {
                VStack {
                    List(dataSource.foundPeripherals) { peripheral in
                        PeripheralCell(peripheral: peripheral) {
                            presenter.didSelect(peripheral: peripheral)
                        }
                    }
                }
                ScanFloatingButton(onTap: presenter.didTapScan)
            }
            .navigationBarTitle("🔍Find BLE Device", displayMode: .inline)
            .toolbar {
                ToolbarItem(placement: .navigationBarLeading) {
                    Button(
                        action: presenter.didTapInfo,
                        label: { Image(systemName: "info.circle") }
                    )
                }
                ToolbarItem(placement: .navigationBarTrailing) {
                    Button(
                        action: presenter.didTapDelete,
                        label: { Image(systemName: "trash") }
                    )
                    .disabled(dataSource.foundPeripherals.count == 0)
                }
            }
            .alert(
                "This is unsupported device.",
                isPresented: $delegate.isUnavailable,
                actions: {
                    Button("OK") {}
                },
                message: { Text("Find the device named \"slate_riiiver\" from the list.") }
            )
            .alert(
                "\(delegate.appInfo.name)",
                isPresented: $delegate.isShowAppInfo,
                actions: {
                    Button("OK") {}
                },
                message: { Text("version: \(delegate.appInfo.version)\nbuild: \(delegate.appInfo.build)") }
            )
        }
        .overlay {
            if delegate.isShowOverlay {
                ZStack {
                    Color.gray.opacity(0.3)
                    ProgressView()
                }
                .ignoresSafeArea()
            }
        }
    }
}

struct DeviceSearchResultView_Previews: PreviewProvider {
    static var previews: some View {
        DeviceSearchResultView(delegate: DeviceSearchResultViewDelegate())
    }
}
