//
//  DeviceSettingsPresenter.swift
//  qcc5100.sample
//

import Foundation

protocol DeviceSettingsPresentation: AnyObject {
    func didAppear() async -> Void
    func onSubmit(of: CommandReceiverCommand, iiideaId: String?) async -> Void
    func didTapBack()
    func didTapRiiiver()
}

final class DeviceSettingsPresenter {
    private let viewDelegate: DeviceSettingsViewInterface
    private let router: DeviceSettingsWireframe
    private let searchDeviceInteractor: SearchDeviceUsecase
    private let manageIiideaInteractor: ManageIiideaUsecase
    
    init(
        viewDelegate: DeviceSettingsViewInterface,
        router: DeviceSettingsWireframe,
        searchDeviceInteractor: SearchDeviceUsecase,
        manageIiideaInteractor: ManageIiideaUsecase
    ) {
        self.viewDelegate = viewDelegate
        self.router = router
        self.searchDeviceInteractor = searchDeviceInteractor
        self.manageIiideaInteractor = manageIiideaInteractor
    }
}

extension DeviceSettingsPresenter: DeviceSettingsPresentation {
    func didAppear() async -> Void {
        print("## [DeviceSettings : Presenter] didAppear: in")
        DispatchQueue.main.async { [self] in
            viewDelegate.toggleOverlay()
        }
        if RiiiverSDKClient.sharedInstance.isLoggedIn {
            do {
                try await manageIiideaInteractor.getMyApplets()
                if let qcc5100iiideas = RiiiverAppletsEntity.sharedInstance.myApplets?.kickedByQCC5100Applets {
                    let iiideas = try await qcc5100iiideas.activatedApplets
                    DispatchQueue.main.async { [self] in
                        iiideas.forEach { viewDelegate.addOptions(PreferredFunctionEntity(displayName: $0.metaData.getTitle(), iiidea: $0)) }
                    }
                }
            } catch(let error) {
                // TODO: error handling
                print(error.localizedDescription)
            }
        }
        DispatchQueue.main.async { [self] in
            viewDelegate.toggleOverlay()
        }
    }
    
    func onSubmit(of: CommandReceiverCommand, iiideaId: String?) async {
        guard let iiideaId else {
            debugPrint("## iiideaId is nil!!")
            return
        }
        let (_, _, activateError) = await manageIiideaInteractor.toggleActivateState(of: iiideaId, newValue: true)
        guard activateError == nil else {
            debugPrint("## activate status error!!")
            return
        }
        manageIiideaInteractor.changeTargetIiidea(of: of, iiideaId: iiideaId)
    }
    
    func didTapBack() {
        searchDeviceInteractor.disconnectPeripheral()
        router.popToSearchDevice()
    }
    
    func didTapRiiiver() {
        router.segueToIiideaList()
    }
}
