//
//  RiiiverLoginPresenter.swift
//  qcc5100.sample
//

import Foundation

protocol RiiiverLoginPresentation: AnyObject {
    func didAppear() async
    func onOpenUrl(_ url: URL) async
    func didTapBack()
}

final class RiiiverLoginPresenter {
    private let viewDelegate: RiiiverLoginViewDelegate
    private let router: RiiiverLoginWireFrame
    private let interactor: LoginRiiiverUsecase
    
    init(
        viewDelegate: RiiiverLoginViewDelegate,
        router: RiiiverLoginWireFrame,
        interactor: LoginRiiiverUsecase
    ) {
        self.viewDelegate = viewDelegate
        self.router = router
        self.interactor = interactor
    }
    
}

extension RiiiverLoginPresenter: RiiiverLoginPresentation {
    func didAppear() async {
        do {
            let url = try await interactor.getLoginUrl()
            viewDelegate.showLoginView(url: url)
        } catch(let error) {
            // TODO: error handling
            print(error.localizedDescription)
        }
    }
    
    func onOpenUrl(_ url: URL) async {
        do {
            try await interactor.verifyCode(of: url)
            router.pop()
        } catch(let error) {
            // TODO: error handling
            print(error.localizedDescription)
        }
    }
    
    func didTapBack() {
        router.popToDeviceSettings()
    }
}
