//
//  RiiiverLoginView.swift
//  qcc5100.sample
//

import SwiftUI
import RiiiverSDK
import AuthenticationServices

protocol RiiiverLoginViewInterface: AnyObject {
    func showLoginView(url: URL)
}

final class RiiiverLoginViewDelegate: ObservableObject, RiiiverLoginViewInterface {
    @Published var isShowLoginScreen: Bool = false
    @Published var loginUrl: URL?
    
    func showLoginView(url: URL) {
        DispatchQueue.main.async {
            self.isShowLoginScreen = true
            self.loginUrl = url
        }
    }
}

struct RiiiverLoginView: View {
    @StateObject private var delegate: RiiiverLoginViewDelegate
    
    var presenter: RiiiverLoginPresentation!
    
    init(delegate: RiiiverLoginViewDelegate) {
        self._delegate = StateObject(wrappedValue: delegate)
    }
    
    var body: some View {
        NavigationView {
            VStack {
                Text("Logging into Riiiver is required")
            }
            .navigationBarTitleDisplayMode(.inline)
            .toolbar {
                ToolbarItem(placement: .navigationBarLeading) {
                    Button("Back") { presenter.didTapBack() }
                }
            }
        }
        .onAppear {
            Task {
                await presenter.didAppear()
            }
        }
        .fullScreenCover(isPresented: $delegate.isShowLoginScreen) {
            if let url = delegate.loginUrl {
                SafariView(url: url).edgesIgnoringSafeArea(.all)
            }
        }
        .onOpenURL { url in
            Task {
                print("## onOpenURL: \(url)")
                await presenter.onOpenUrl(url)
            }
        }
    }
}
