//
//  ProfilePresenter.swift
//  qcc5100.sample
//

import Foundation
import RiiiverSDK

protocol UserWebViewPresentation: AnyObject, ERUserWebPagesViewDelegate {
    func didAppear() async -> Void
    func didTapBack()
    func onDismisExternalView()
}

final class UserWebViewPresenter {
    private let viewDelegate: UserWebViewDelegate
    private let subViewDelegate: UserWebViewWrapperDelegate
    private let router: UserWebViewWireframe
    private let interactor: ManageIiideaUsecase
    private let requestPermissionInteractor: RequestPermissionUsecase
    
    init(
        viewDelegate: UserWebViewDelegate,
        subViewDelegate: UserWebViewWrapperDelegate,
        router: UserWebViewWireframe,
        interactor: ManageIiideaUsecase,
        requestPermissionInteractor: RequestPermissionUsecase
    ) {
        self.viewDelegate = viewDelegate
        self.subViewDelegate = subViewDelegate
        self.router = router
        self.interactor = interactor
        self.requestPermissionInteractor = requestPermissionInteractor
    }
}

extension UserWebViewPresenter: UserWebViewPresentation {
    func didAppear() async {
        print("## UserWebViewPresenter:didAppear: in")
        if !RiiiverSDKClient.sharedInstance.isLoggedIn {
            router.segueLoginView()
        } else {
            do {
                try await interactor.getMyApplets()
            } catch(let error) {
                print("getMyApplets Failed!: \(error.localizedDescription)")
            }
        }
    }
    
    func didTapBack() {
        router.popToMyIiidea()
    }
    
    func onDismisExternalView() {
        print("## left from the external view")
        subViewDelegate.toggleWaitState()
    }
}

extension UserWebViewPresenter: ERUserWebPagesViewDelegate {
    func didReceivePermissionRequest(permissionType: RiiiverSDK.ERPermissionType) {
        requestPermissionInteractor.requestOsPermission(permissionType.rawValue)
    }
    
    func didReceiveOpenUrlRequest(url: String) {
        subViewDelegate.toggleWaitState()
        viewDelegate.openUrl(url: URL(string:url)!)
    }
    
    func didReceiveCloseRequest() {
        router.popToMyIiidea()
    }
    
    func onError(error: RiiiverSDK.ERError) {
        print("## error!: \(error.errorMessage)")
    }
    
    func onLogout() {
        //
    }
}
