//
//  UserWebViewWrapper.swift
//  qcc5100.sample
//

import SwiftUI
import WebKit
import RiiiverSDK

enum UserWebPageContent {
    case top
    case settings(iiideaId: String)
    case store
    case personalInfo(iiideaId: String)
}

protocol UserWebViewWrapperInterface: AnyObject {
    func toggleWaitState()
}

final class UserWebViewWrapperDelegate: ObservableObject, UserWebViewWrapperInterface {
    @Published var isWaitingExternalView: Bool = false

    func toggleWaitState() {
        DispatchQueue.main.async {
            self.isWaitingExternalView.toggle()
        }
    }
}

struct UserWebViewWrapper: UIViewRepresentable {    
    typealias UIViewType = RiiiverSDK.ERUserWebPagesView
    
    @StateObject var viewDelegate: UserWebViewWrapperDelegate
    let content: UserWebPageContent
    let delegate: ERUserWebPagesViewDelegate
    
    init(
        content: UserWebPageContent,
        delegate: ERUserWebPagesViewDelegate,
        viewDelegate: UserWebViewWrapperDelegate
    ) {
        self._viewDelegate = StateObject(wrappedValue: viewDelegate)
        self.content = content
        self.delegate = delegate
    }
    
    func makeUIView(context: Context) -> RiiiverSDK.ERUserWebPagesView {
        let uiView = RiiiverSDK.ERUserWebPagesView()
        uiView.delegate = self.delegate
        return uiView
    }
    
    func updateUIView(_ uiView: UIViewType, context: Context) {
        uiView.dispose()
        switch self.content {
        case .settings(let iiideaId):
            uiView.loadPage(type: .appletSettings(deviceId: RiiiverConstants.deviceIds.first!, appletId: iiideaId))
        case .store:
            uiView.loadPage(type: .store(deviceId: RiiiverConstants.deviceIds.first!))
        case .personalInfo(let iiideaId):
            uiView.loadPage(type: .personalInfo(deviceId: RiiiverConstants.deviceIds.first!, appletId: iiideaId))
        case .top: fallthrough
        default:
            uiView.loadPage(type: .top(deviceId: RiiiverConstants.deviceIds.first!))
        }
    }
    
    static func dismantleUIView(_ uiView: ERUserWebPagesView, coordinator: ()) {
        uiView.dispose()
    }
}
