//
//  UserWebView.swift
//  qcc5100.sample
//

import RiiiverSDK
import SwiftUI

protocol UserWebViewInterface: AnyObject {
    func openUrl(url: URL)
}

final class UserWebViewDelegate: ObservableObject {
    @Published var isShowWebPage: Bool = false
    @Published var url: URL?

    func openUrl(url: URL) {
        self.isShowWebPage = true
        self.url = url
    }
}

struct UserWebView: View {
    @StateObject var viewDelegate: UserWebViewDelegate
    @StateObject var subViewDelegate: UserWebViewWrapperDelegate
    var presenter: UserWebViewPresentation!
    
    init(viewDelegate: UserWebViewDelegate, subViewDelegate: UserWebViewWrapperDelegate) {
        self._viewDelegate = StateObject(wrappedValue: viewDelegate)
        self._subViewDelegate = StateObject(wrappedValue: subViewDelegate)
    }
    
    var body: some View {
        NavigationView {
            VStack {
                UserWebViewWrapper(content: .top, delegate: presenter, viewDelegate: subViewDelegate)
            }
            .navigationBarTitleDisplayMode(.inline)
            .toolbar {
                ToolbarItem(placement: .navigationBarLeading) {
                    Button("Back") { presenter.didTapBack() }
                }
            }
            .onAppear {
                Task {
                    await presenter.didAppear()
                }
            }
        }
        .fullScreenCover(isPresented: $viewDelegate.isShowWebPage, onDismiss: presenter.onDismisExternalView) {
            if let url = viewDelegate.url {
                SafariView(url: url).edgesIgnoringSafeArea(.all)
            }
        }
    }
}
